package com.lutao.uav.mqtt;

import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.lutao.uav.domain.entity.NorthSeaGasPoint;
import com.lutao.uav.service.NorthSeaGasPointService;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.paho.client.mqttv3.*;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.geo.Circle;
import org.springframework.data.geo.Distance;
import org.springframework.data.geo.GeoResults;
import org.springframework.data.geo.Point;
import org.springframework.data.redis.connection.RedisGeoCommands;
import org.springframework.data.redis.core.BoundGeoOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.kafka.core.KafkaTemplate;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.UUID;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

@Component
@Slf4j
public class MqttConfig {

    @Value("${north-sea-mqtt.broker-url}")
    private String brokerUrl;

    @Value("${north-sea-mqtt.username}")
    private String username;

    @Value("${north-sea-mqtt.password}")
    private String password;

    @Value("${north-sea-mqtt.topic}")
    private String topic;

    @Resource(name = "redisTemplateLettuce")
    private RedisTemplate redisTemplate;

    @Resource
    private NorthSeaGasPointService northSeaGasPointService;

    @Resource
    private KafkaTemplate kafkaTemplate;

    private MqttClient client;
    private ExecutorService executorService;

    @PostConstruct
    public void init() {
        log.info("-------------------初始化MQTT配置-------------------");
        //initPointData();
        executorService = Executors.newSingleThreadExecutor();
        connectWithReconnect();
        //test();
    }

    //初始化点位数据
    public void initPointData() {
        log.info("-------------------初始化北海点位数据-------------------");
        List<NorthSeaGasPoint> northSeaGasPoints = northSeaGasPointService.list();
        HashMap<Object, Object> map = new HashMap<>();
        BoundGeoOperations northSeaPoint = redisTemplate.boundGeoOps("north_sea_point");
        northSeaGasPoints.forEach(northSeaGasPoint -> {
            String jsonStr = JSONUtil.toJsonStr(northSeaGasPoint);
            map.put(jsonStr, new Point(Double.parseDouble(northSeaGasPoint.getLongitude()), Double.parseDouble(northSeaGasPoint.getLatitude())));
        });
        northSeaPoint.add(map);
        log.info("-------------------初始化成功-------------------");
    }

    //初始化数据库
    public void initDatabase() {
        boolean remove = northSeaGasPointService.remove(null);
        log.info("-------------------初始化北海点位数据库-------------------");
        NorthSeaGasPoint northSeaGasPoint = new NorthSeaGasPoint();
        northSeaGasPoint.setId(UUID.randomUUID().toString());
        northSeaGasPoint.setPointName("氨气");
        northSeaGasPoint.setPointCode("wrj-aq");
        northSeaGasPoint.setPointCode2("wrj-aq");
        northSeaGasPoint.setPointType("氨气");
        northSeaGasPoint.setUnit("ppm");
        northSeaGasPoint.setHighReport("26");
        northSeaGasPoint.setHighHighReport("52");

        NorthSeaGasPoint northSeaGasPoint1 = new NorthSeaGasPoint();
        northSeaGasPoint1.setId(UUID.randomUUID().toString());
        northSeaGasPoint1.setPointName("甲烷");
        northSeaGasPoint1.setPointCode("wrj-jw");
        northSeaGasPoint1.setPointCode2("wrj-jw");
        northSeaGasPoint1.setPointType("甲烷");
        northSeaGasPoint1.setUnit("%LEL");
        northSeaGasPoint1.setHighReport("25");
        northSeaGasPoint1.setHighHighReport("50");

        NorthSeaGasPoint northSeaGasPoint2 = new NorthSeaGasPoint();
        northSeaGasPoint2.setId(UUID.randomUUID().toString());
        northSeaGasPoint2.setPointName("硫化氢");
        northSeaGasPoint2.setPointCode("wrj-liuhq");
        northSeaGasPoint2.setPointCode2("wrj-liuhq");
        northSeaGasPoint2.setPointType("硫化氢");
        northSeaGasPoint2.setUnit("PPM");
        northSeaGasPoint2.setHighReport("2.5");
        northSeaGasPoint2.setHighHighReport("5");

        NorthSeaGasPoint northSeaGasPoint3 = new NorthSeaGasPoint();
        northSeaGasPoint3.setId(UUID.randomUUID().toString());
        northSeaGasPoint3.setPointName("一氧化碳");
        northSeaGasPoint3.setPointCode("wrj-yyht");
        northSeaGasPoint3.setPointCode2("wrj-yyht");
        northSeaGasPoint3.setPointType("一氧化碳");
        northSeaGasPoint3.setUnit("PPM");

        NorthSeaGasPoint northSeaGasPoint4 = new NorthSeaGasPoint();
        northSeaGasPoint4.setId(UUID.randomUUID().toString());
        northSeaGasPoint4.setPointName("二氧化氮");
        northSeaGasPoint4.setPointCode("wrj-eyhd");
        northSeaGasPoint4.setPointCode2("wrj-eyhd");
        northSeaGasPoint4.setPointType("二氧化氮");
        northSeaGasPoint4.setUnit("PPM");
        northSeaGasPoint4.setHighReport("2.4");
        northSeaGasPoint4.setHighHighReport("4.8");

        NorthSeaGasPoint northSeaGasPoint5 = new NorthSeaGasPoint();
        northSeaGasPoint5.setId(UUID.randomUUID().toString());
        northSeaGasPoint5.setPointName("二氧化硫");
        northSeaGasPoint5.setPointCode("wrj-eyhl");
        northSeaGasPoint5.setPointCode2("wrj-eyhl");
        northSeaGasPoint5.setPointType("二氧化硫");
        northSeaGasPoint5.setUnit("PPM");

        NorthSeaGasPoint northSeaGasPoint6 = new NorthSeaGasPoint();
        northSeaGasPoint6.setId(UUID.randomUUID().toString());
        northSeaGasPoint6.setPointName("氯化氢");
        northSeaGasPoint6.setPointCode("wrj-lvhq");
        northSeaGasPoint6.setPointCode2("wrj-lvhq");
        northSeaGasPoint6.setPointType("氯化氢");
        northSeaGasPoint6.setUnit("PPM");
        northSeaGasPoint4.setHighReport("2.5");
        northSeaGasPoint4.setHighHighReport("5");

        boolean b = northSeaGasPointService.saveBatch(Arrays.asList(northSeaGasPoint, northSeaGasPoint1, northSeaGasPoint2, northSeaGasPoint3, northSeaGasPoint4, northSeaGasPoint5, northSeaGasPoint6));
        if (b){
            log.info("-------------------初始化成功-------------------");
            initPointData();
        }else {
            log.info("-------------------初始化失败-------------------");
        }

    }

    private NorthSeaGasPoint createGasPoint(String pointName, String pointCode, String pointType, String unit, String highReport, String highHighReport) {
        NorthSeaGasPoint gasPoint = new NorthSeaGasPoint();
        gasPoint.setId(UUID.randomUUID().toString());
        gasPoint.setPointName(pointName);
        gasPoint.setPointCode(pointCode);
        gasPoint.setPointCode2(pointCode);
        gasPoint.setPointType(pointType);
        gasPoint.setUnit(unit);
        if (highReport != null) gasPoint.setHighReport(highReport);
        if (highHighReport != null) gasPoint.setHighHighReport(highHighReport);
        return gasPoint;
    }

    private void connectWithReconnect() {
        new Thread(() -> {
            while (!Thread.currentThread().isInterrupted()) {
                try {
                    MqttConnectOptions options = new MqttConnectOptions();
                    options.setCleanSession(true);
                    options.setKeepAliveInterval(10);
                    options.setConnectionTimeout(30);
                    options.setUserName(username);
                    options.setPassword(password.toCharArray());

                    client = new MqttClient(brokerUrl, MqttClient.generateClientId());
                    client.setCallback(new MqttCallback() {
                        @Override
                        public void connectionLost(Throwable cause) {
                            log.info("MQTT连接丢失，尝试重连:{}", cause.getMessage());
                            // 重置客户端实例，以便重新连接
                            client = null;
                            connectWithReconnect(); // 递归调用重连方法
                        }

                        @Override
                        public void messageArrived(String topic, MqttMessage message) throws Exception {
                            String content = new String(message.getPayload());
                            log.info("接收到主题:{}的消息{}", topic, content);
                            if (JSONUtil.isJson(content)) {
                                //handleJsonMessage(JSONObject.parseObject(content));
                                JSONObject jsonObject = JSONObject.parseObject(content);
                                double longitude = jsonObject.getDoubleValue("longitude");
                                double latitude = jsonObject.getDoubleValue("latitude");
                                try {
                                    /**
                                     * {
                                     * 	"enterpriseId":"140170010",
                                     * "equipCode":"141110055001G0001",
                                     * "surveyInstrumentCode":"141110055001G0001YL001",
                                     * "monitorType":"6",
                                     * "monitorUnit":"mm",
                                     * "realtimeValue":"1.57",
                                     * "collectTime":"2024-04-12 18:21:11"
                                     * }
                                     */
                                    List<NorthSeaGasPoint> gasMessages = northSeaGasPointService.list();
                                    for (NorthSeaGasPoint gasMessage : gasMessages) {
                                        JSONObject kafkaSendMessage = new JSONObject();
                                        kafkaSendMessage.put("enterpriseId", gasMessage.getCompanyName());
                                        kafkaSendMessage.put("equipCode", gasMessage.getPointCode());
                                        kafkaSendMessage.put("surveyInstrumentCode", gasMessage.getPointCode2());
                                        kafkaSendMessage.put("monitorUnit", gasMessage.getUnit());
                                        String utcTimeStr = jsonObject.getString("utcTime");
                                        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd-HH-mm-ss");
                                        LocalDateTime localDateTime = LocalDateTime.parse(utcTimeStr, formatter);

                                        // 将LocalDateTime转换为Instant
                                        Instant instant = localDateTime.atZone(ZoneId.of("UTC")).toInstant();

                                        // 转换到东八区
                                        ZonedDateTime shanghaiTime = instant.atZone(ZoneId.of("Asia/Shanghai"));

                                        // 格式化为字符串
                                        String formattedTime = shanghaiTime.format(DateTimeFormatter.ISO_LOCAL_DATE_TIME);

                                        // 定义输入和输出的日期时间格式
                                        DateTimeFormatter inputFormat = DateTimeFormatter.ofPattern("yyyy-MM-dd'T'HH:mm:ss");
                                        DateTimeFormatter outputFormat = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss");

                                        // 解析原始字符串到LocalDateTime对象
                                        LocalDateTime dateTime = LocalDateTime.parse(formattedTime, inputFormat);

                                        // 将LocalDateTime对象格式化为新的字符串格式
                                        String formattedDateTime = dateTime.format(outputFormat);

                                        // 输出转换后的字符串
                                        kafkaSendMessage.put("collectTime", formattedDateTime);

                                        // kafkaSendMessage.put("realtimeValue", sendMessage.get("unit"));

                                        //NH3/NO2/CH4/H2S/SO2/HCL/CO
                                        //3 有毒气体  4 可燃气体
                                        String pointType = gasMessage.getPointType();
                                        log.info("气体类型{}", pointType);
                                        if ("氨气".equals(pointType)) { //可燃 同时有毒
                                            String NH3 = jsonObject.getJSONObject("airData").getString("NH3(ppm)");
                                            log.info("气体{},内容{}", pointType, kafkaSendMessage);
                                            kafkaSendMessage.put("monitorType", "4");
                                            kafkaSendMessage.put("realtimeValue", NH3);
                                            if (StringUtils.isBlank(NH3)){
                                                continue;
                                            }
                                            try {
                                                kafkaTemplate.send("majorHazard", kafkaSendMessage.toJSONString());
                                                log.info("发送了 majorHazard 消息,气体{},内容{}", pointType, kafkaSendMessage);
                                                kafkaSendMessage.put("monitorType", "3");
                                                kafkaTemplate.send("majorHazard", kafkaSendMessage.toJSONString());
                                                log.info("发送了 majorHazard 消息,气体{},内容{}", pointType, kafkaSendMessage);
                                            } catch (Exception e) {
                                                log.error("发送消息失败,气体{},内容{}", pointType, kafkaSendMessage);
                                            }
                                        }

                                        if ("甲烷".equals(pointType)) {
                                            String CH4 = jsonObject.getJSONObject("airData").getString("CxHy(%)");
                                            kafkaSendMessage.put("monitorType", "4");
                                            kafkaSendMessage.put("realtimeValue", CH4);
                                            if (StringUtils.isBlank(CH4)){
                                                continue;
                                            }
                                            kafkaTemplate.send("majorHazard", kafkaSendMessage.toJSONString());
                                            log.info("发送了 majorHazard 消息,气体{},内容{}", pointType, kafkaSendMessage);
                                        }

                                        if ("硫化氢".equals(pointType)) { //可燃 同时有毒
                                            String H2S = jsonObject.getJSONObject("airData").getString("H2S(ppm)");
                                            kafkaSendMessage.put("monitorType", "4");
                                            kafkaSendMessage.put("realtimeValue", H2S);
                                            if (StringUtils.isBlank(H2S)){
                                                continue;
                                            }
                                            kafkaTemplate.send("majorHazard", kafkaSendMessage.toJSONString());
                                            log.info("发送了 majorHazard 消息,气体{},内容{}", pointType, kafkaSendMessage);
                                            kafkaSendMessage.put("monitorType", "3");
                                            kafkaTemplate.send("majorHazard", kafkaSendMessage.toJSONString());
                                            log.info("发送了 majorHazard 消息,气体{},内容{}", pointType, kafkaSendMessage);
                                        }

                                        if ("一氧化碳".equals(pointType)) { //可燃 同时有毒
                                            String CO = jsonObject.getJSONObject("airData").getString("CO(ppm)");
                                            kafkaSendMessage.put("monitorType", "4");
                                            kafkaSendMessage.put("realtimeValue", CO);
                                            if (StringUtils.isBlank(CO)){
                                                continue;
                                            }
                                            kafkaTemplate.send("majorHazard", kafkaSendMessage.toJSONString());
                                            log.info("发送了 majorHazard 消息,气体{},内容{}", pointType, kafkaSendMessage);
                                            kafkaSendMessage.put("monitorType", "3");
                                            kafkaTemplate.send("majorHazard", kafkaSendMessage.toJSONString());
                                            log.info("发送了 majorHazard 消息,气体{},内容{}", pointType, kafkaSendMessage);
                                        }

                                        if ("二氧化氮".equals(pointType)) {
                                            String NO2 = jsonObject.getJSONObject("airData").getString("NO2(ppm)");
                                            kafkaSendMessage.put("monitorType", "3");
                                            kafkaSendMessage.put("realtimeValue", NO2);
                                            if (StringUtils.isBlank(NO2)){
                                                continue;
                                            }
                                            kafkaTemplate.send("majorHazard", kafkaSendMessage.toJSONString());
                                            log.info("发送了 majorHazard 消息,气体{},内容{}", pointType, kafkaSendMessage);
                                        }

                                        if ("二氧化硫".equals(pointType)) {
                                            String SO2 = jsonObject.getJSONObject("airData").getString("SO2(ppm)");
                                            kafkaSendMessage.put("monitorType", "3");
                                            kafkaSendMessage.put("realtimeValue", SO2);
                                            if (StringUtils.isBlank(SO2)){
                                                continue;
                                            }
                                            // TODO: 2024/6/24 kafka  send
                                            kafkaTemplate.send("majorHazard", kafkaSendMessage.toJSONString());
                                            log.info("发送了 majorHazard 消息,气体{},内容{}", pointType, kafkaSendMessage);
                                        }

                                        if ("氯化氢".equals(pointType)) {
                                            String HCL = jsonObject.getJSONObject("airData").getString("HCL(ppm)");
                                            kafkaSendMessage.put("monitorType", "3");
                                            kafkaSendMessage.put("realtimeValue", HCL);
                                            if (StringUtils.isBlank(HCL)){
                                                continue;
                                            }
                                            kafkaTemplate.send("majorHazard", kafkaSendMessage.toJSONString());
                                            log.info("发送了 majorHazard 消息,气体{},内容{}", pointType, kafkaSendMessage);
                                        }
                                    }


                                } catch (Exception e) {
                                    e.printStackTrace();
                                    log.error("MqttClient receive error:{}", e.getMessage());
                                }
                            }
                        }

                        @Override
                        public void deliveryComplete(IMqttDeliveryToken token) {
                            // 消息已送达
                        }
                    });

                    client.connect(options);
                    client.subscribe(topic);
                    log.info("-------------------MQTT客户端已连接并订阅主题:{}-------------------", topic);
                    break; // 连接成功，退出循环
                } catch (MqttException e) {
                    //System.out.println("MQTT连接失败，3秒后重试: " + e.getMessage());
                    log.info("MQTT连接失败，3秒后重试:{}", e.getMessage());
                    try {
                        Thread.sleep(3000); // 等待3秒
                    } catch (InterruptedException e1) {
                        Thread.currentThread().interrupt(); // 重新设置中断状态
                        return;
                    }
                }
            }
        }).start();
    }

    public void test() {
        Point point = new Point(0, 0);
        Distance distance = new Distance(1000000, RedisGeoCommands.DistanceUnit.KILOMETERS);
        Circle circle = new Circle(point, distance);

        RedisGeoCommands.GeoRadiusCommandArgs args = RedisGeoCommands.GeoRadiusCommandArgs.newGeoRadiusArgs()
                .sortAscending()
                .includeCoordinates()
                .includeDistance()
                .limit(1);

        GeoResults<RedisGeoCommands.GeoLocation<String>> outlets = redisTemplate.boundGeoOps("north_sea_point").radius(circle, args);
        outlets.forEach(outlet -> {
            log.info("需要传送的消息: {}", outlet.getContent().getName());
        });
    }

    //@Scheduled(fixedRate = 2000)
    public void testsend() {
        kafkaTemplate.send("majorHazard", "测试消息");
        log.info("发送了 majorHazard 消息");
    }

    @PreDestroy
    public void destroy() {
        if (client != null && client.isConnected()) {
            try {
                client.disconnect();
            } catch (MqttException e) {
                e.printStackTrace();
            }
        }
        if (executorService != null) {
            executorService.shutdownNow();
        }
    }
}
